#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-security-utils-config v. 1.2.3
#
# Post-installation script for configuring the gLite Security Utilities
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
#
# Version info: $Id$
#
# Usage: python glite-security-utils-config [-c|-v|-h|--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#        --configure configure the security utilities
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,re,pwd
from shutil import copy
import sys, posix
import getopt
import time, random

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams
import mkgridmap

# Set global variables here 
global params                         # all config values from the XML file

class glite_security_utils:

    def __init__(self):
        self.verbose = 0
        
        self.version = "1.2.3"
        self.name = "glite-security-utils"
        self.friendly_name = "gLite Security Utilities"
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the security utilities (default option if none is specified)'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        
        error_level = 0
        
        if params["install.fetch-crl.cron"] == "true":

            if os.geteuid() == 0:
                
                # Add random delay
                if params['fetch-crl.cron.random.delay'] == "true":
                    random.seed()
                    delay = int(params['fetch-crl.cron.tab'].strip().split(" ",1)[0]) + random.randrange(1,30)
                    if delay >= 60: delay = delay - 60
                    crontime = "%d %s" \
                        % (delay, params['fetch-crl.cron.tab'].strip().split(" ",1)[1])
                else:
                    crontime = params['fetch-crl.cron.tab']
            
                print "Installing Fetch CRL crontab as /etc/cron.d/glite-fetch-crl.cron...",
        
                crontab = "/etc/cron.d/glite-fetch-crl.cron"
                cronjob = "%s root sh %s -o %s -q >> %s/glite-fetch-crl-cron.log 2>&1\n\n" \
                    % (crontime, params['fetch-crl.script'], params['ca.certificates.dir'],os.environ['GLITE_LOCATION_LOG'])
    
                if not os.path.exists(crontab):
                    os.system("/bin/touch %s" % crontab)
                os.chmod(crontab,0644)
                f = open(crontab,'w')
                f.write("PATH=%s/sbin:/sbin:/bin:/usr/sbin:/usr/bin\n" % params['EDG_LOCATION'])
                f.write("HOME=/\n")
                f.write("HOSTNAME=%s\n" % os.environ['HOSTNAME'])
                f.write("EDG_LOCATION=%s\n" % params['EDG_LOCATION'])
                f.write("MAILTO=%s\n\n" % params['cron.mailto'])
                f.write(cronjob)
                f.close()
            
                glib.printOkMessage()
                   
                # Run it once if never done
                for file in os.listdir(params['ca.certificates.dir']):
                    if file.find('.r0') != -1:
                        break
                else:
                    print "Updating CRLs...                                         ",
                    if self.verbose:
                        os.system("%s -o %s" \
                            % (params['fetch-crl.script'], params['ca.certificates.dir']))
                    else:
                        os.system("%s -o %s -q" \
                            % (params['fetch-crl.script'], params['ca.certificates.dir']))
            
            else:

                print "Configuring CRL..."

                installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
                
                # installation to the users crontab. TBD
                
                
                # Run it once
                for file in os.listdir(params['ca.certificates.dir']):
                    if file.find('.r0') != -1:
                        break
                else:
                    print "Updating CRLs                                            ",
                    if self.verbose:
                        os.system("%s%s -o %s" \
                            % (installPrefix, params['fetch-crl.script'], params['ca.certificates.dir']))
                    else:
                        os.system("%s%s -o %s -q" \
                            % (installPrefix, params['fetch-crl.script'], params['ca.certificates.dir']))
            

            glib.printOkMessage()
            
        else:
            
            crontab = "/etc/cron.d/glite-fetch-crl.cron"
            if os.path.exists(crontab):
                print "Removing fetch-crl crontab /etc/cron.d/glite-fetch-crl.cron...",
                os.remove(crontab)
                glib.printOkMessage()
           

        if params["install.mkgridmap.cron"] == "true": 
        
            print "Installing mkgridmap crontab as /etc/cron.d/glite-mkgridmap.cron...",

            crontab = "/etc/cron.d/glite-mkgridmap.cron"
            cronjob = "%s root python %s/etc/config/scripts/mkgridmap.py > /dev/null\n\n" \
                % (params['mkgridmap.cron.tab'],os.environ['GLITE_LOCATION'])

            if not os.path.exists(crontab):
                os.system("/bin/touch %s" % crontab)
            os.chmod(crontab,0644)
            f = open(crontab,'w')
            f.write("PATH=/sbin:/bin:/usr/sbin:/usr/bin\n")
            f.write("HOME=/\n")
            f.write("HOSTNAME=%s\n" % os.environ['HOSTNAME'])
            f.write("MAILTO=%s\n\n" % params['cron.mailto'])
            f.write(cronjob)
            f.close()
        
            glib.printOkMessage()

            print "Run mkgridmap once to update the file...",
            
            return_result = os.system("python %s/mkgridmap.py" % glib.getScriptPath())
            if return_result == 1:
                return 1
            elif return_result > 1:
                error_level = 2
        
            glib.printOkMessage()
            
        else:
            
            crontab = "/etc/cron.d/glite-mkgridmap.cron"
            if os.path.exists(crontab):
                print "Removing mkgridmap crontab /etc/cron.d/glite-mkgridmap.cron...",
                os.remove(crontab)
                glib.printOkMessage()

        # Touch cron spool directory to cause reloading of the crontabs
        if os.geteuid() == 0:
            os.system("/bin/touch /var/spool/cron")

        return error_level
    
def loadDefaults(params):

    params['GLITE_LOCATION'] = "/opt/glite"
    params['EDG_LOCATION'] = "/opt/edg"
    params['install.fetch-crl.cron'] = "true"
    params['fetch-crl.cron.tab'] = "00 */6 * * *"
    params['install.mkgridmap.cron'] = "false"
    params['fetch-crl.cron.random.delay'] = "true"
    params['fetch-crl.script'] = "%s/sbin/edg-fetch-crl" % params['EDG_LOCATION']
    params['mkgridmap.script'] = "%s/sbin/edg-mkgridmap" % params['EDG_LOCATION']
    params['mkgridmap.conf'] = "%s/etc/glite-mkgridmap.conf" % params['GLITE_LOCATION']

#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    if not os.geteuid() == 0:
        
        params['installer.export.filename'] = glib.checkSetEnvFile(params['installer.export.filename'])
        glib.printInfoMessage("gLite environment file will be saved to %s" % params['installer.export.filename'])
        
        installPrefix = os.path.abspath(glib.getScriptPath() + "/../../../")
        
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/lib' % installPrefix)
        glib.addEnvPath('LD_LIBRARY_PATH','%s/usr/local/lib' % installPrefix)
        
        glib.addEnvPath('PATH','%s/usr/bin' % installPrefix)
        
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/site_perl/5.6.0/" % installPrefix)
        glib.addEnvPath('PERL5LIB', "%s/usr/lib/perl5/vendor_perl/5.8.0/" % installPrefix)
        
        glib.printWarningMessage("The configuration script can modify the environment variables")
        print "    Modifications are listed below\n"
        
        if os.path.exists(params['GLITE_LOCATION']):
            if not os.access(params['GLITE_LOCATION'], os.W_OK):
                print "    GLITE_LOCATION is set to the non-writable directory %s -" % params['GLITE_LOCATION'],
                params['GLITE_LOCATION'] = "%s" % (installPrefix)
                print "it will be changed to %s" % params['GLITE_LOCATION']
            else:
                if not params['GLITE_LOCATION'] == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION'] = "%s" % (installPrefix)

        if not os.path.exists(params['GLOBUS_LOCATION']):
            print "    GLOBUS_LOCATION is set to the non-existing directory %s " % (params['GLOBUS_LOCATION']),
            params['GLOBUS_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GLOBUS_LOCATION']

        if not os.path.exists(params['GPT_LOCATION']):
            print "    GPT_LOCATION is set to the non-existing directory %s " % (params['GPT_LOCATION']),
            params['GPT_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['GPT_LOCATION']

        if not os.path.exists(params['EDG_LOCATION']):
            print "    EDG_LOCATION is set to the non-existing directory %s " % (params['EDG_LOCATION']),
            params['EDG_LOCATION'] = "%s" % (installPrefix)
            print "it will be changed to %s" % params['EDG_LOCATION']

        if os.path.exists(params['GLITE_LOCATION_VAR']):
            if not os.access('GLITE_LOCATION_VAR', os.W_OK):
                print "    GLITE_LOCATION_VAR is set to the non-writable directory %s - " % params['GLITE_LOCATION_VAR'],
                params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
                print "value changed to %s" % params['GLITE_LOCATION_VAR']
            else:
                if not params['GLITE_LOCATION_VAR'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_VAR'] = "%s/var/glite" % installPrefix
        
        if os.path.exists(params['GLITE_LOCATION_LOG']):
            if not os.access('GLITE_LOCATION_LOG', os.W_OK):
                print "    GLITE_LOCATION_LOG is set to the non-writable directory %s - " % params['GLITE_LOCATION_LOG'],
                params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
                print "it will be changed to %s" % params['GLITE_LOCATION_LOG']
            else:
                if not params['GLITE_LOCATION_LOG'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['GLITE_LOCATION_LOG'] = "%s/var/log/glite" % installPrefix
        
        if os.path.exists(params['ca.certificates.dir']):
            if not os.access('ca.certificates.dir', os.W_OK):
                print "    ca.certificates.dir is set to the non-writable directory %s - " % params['ca.certificates.dir'],
                params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
                print "it will be changed to %s" % params['ca.certificates.dir']
            else:
                if not params['ca.certificates.dir'].startswith == installPrefix:
                    glib.printWarningMessage("The configuration file possibly points to another gLite UI installation for this same user")
                    glib.printWarningMessage("Please verify the configuration in the glite-global.cfg.xml file before proceeding")
                    sys.exit(1)
        else:
            params['ca.certificates.dir'] = "%s/etc/grid-security/certificates" % installPrefix
        
        print "\n"

    # Standard configuration
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    glib.export('GLITE_LOCATION_LOG');
    glib.export('GLITE_LOCATION_TMP');
    
    glib.check_dir(os.environ['GLITE_LOCATION_VAR'])
    glib.check_dir(os.environ['GLITE_LOCATION_LOG'])
    glib.check_dir(os.environ['GLITE_LOCATION_TMP'])
    
    glib.export('GLOBUS_LOCATION');
    glib.export('GPT_LOCATION');
    glib.export('EDG_LOCATION');

    # bin and lib paths
    glib.addEnvPath("PATH","$GLITE_LOCATION/externals/bin")
    glib.addEnvPath("PATH","$GLITE_LOCATION/bin")

    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/externals/lib")
    glib.addEnvPath("LD_LIBRARY_PATH","$GLITE_LOCATION/lib")

    # MANPATH
    glib.addEnvPath('MANPATH', '$GLOBUS_LOCATION/man')
    glib.addEnvPath('MANPATH', '$GLITE_LOCATION/share/man')

    # Perl
    glib.addEnvPath("PERL5LIB", "$GPT_LOCATION/lib/perl")
    glib.addEnvPath("PERL5LIB", "$GLITE_LOCATION/lib/perl5")

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
#    if not os.geteuid()==0:
#        print '"\nThis script must be run as root\n'
#        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    loadDefaults(params)
    try:
        opts, args = getopt.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration("%s/../glite-security-utils.cfg.xml" % glib.getScriptPath(),params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_security_utils()
    service.verbose = verbose
    
    # Command line opts if any
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'configure', 'siteconfig=', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    subservice=0

    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o=="--subservice":
            subservice=1
        if o=="--configure":
            pass

    # Print configuration parameters
    if verbose:
        glib.print_params(params)

    if (subservice ==0):
        service.copyright()
        service.showVersion()
        service.banner()
        
    # Configure the service
    configuration_result = service.configure()
    if configuration_result == 0:
        print "\n%s configuration successfully completed\n" \
            % service.friendly_name
    elif configuration_result == 2:
        print "\nOne or more warnings were issued while configuring the %s service" \
            % service.friendly_name
        print "Please revise them before continuing"
        sys.exit(2)
    else:
        print "\nAn unrecoverable error occurred while configuring the %s service" \
            % service.friendly_name
        sys.exit(1)
    
