#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-mkgridmap v. 1.2.3
#
# Script for handling updates of gridmap-file using glite-mkgridmap
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
#
# Version info: $Id: mkgridmap.py,v 1.11.2.3 2006/02/21 21:13:53 dimeglio Exp $
#
# Usage: python glite-mkgridmap [-c|-v|-h|--help]
#        -c          print configuration
#        -v          print version
#        -h,--help   print usage info
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,re,pwd
from shutil import copy
import sys, posix
import getopt
import time

from  gLiteInstallerLib import gLib
from  gLiteInstallerLib import ConfigParams

# Set global variables here 
global params                         # all config values from the XML file
global glib

class glite_mkgridmap:

    def __init__(self,params={}):
        self.verbose = 0
        
        self.version = "1.2.3"
        self.name = "glite-mkgridmap"
        self.friendly_name = "gLite MKGridMap Utility"
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "%s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Run the script without options to update the \
gridmap file preserving custom entries \
or run it with the following options:\n\n 
python %s [OPTION...]""" % self.name, os.environ['GLITE_LOCATION']

        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        
        error_level = 0
        
        if not params.has_key('mkgridmap.conf'):
            params['mkgridmap.conf'] = "%s/etc/glite-mkgridmap.conf" % os.environ['GLITE_LOCATION']
        if not params.has_key('mkgridmap.script'):
            params['mkgridmap.script'] = "/opt/edg/sbin/edg-mkgridmap"

        confFile = params['mkgridmap.conf']
        mkgFile = params['mkgridmap.script']

        print "Running the %s..." % self.friendly_name
        
        # Check if mkgridmap configuration file is valid
        if not os.path.exists(confFile):
            os.system("/bin/touch %s" % confFile)
        f = open(confFile,'r')
        for line in f.xreadlines():
            if not line.strip().startswith('#') and line != "":
                break
        else:
            glib.printWarningMessage("The %s file doesn't seem to contain any valid information" % confFile)
            glib.printWarningMessage("Please check it and re-run this script or run")
            glib.printWarningMessage("%s manually" % mkgFile)
            error_level = 2
        f.close()

        # Save existing file if any
        if os.path.exists("%s.1" % params['host.gridmapfile']):
            os.remove("%s.1" % params['host.gridmapfile'])
        if os.path.exists(params['host.gridmapfile']):
            os.rename(params['host.gridmapfile'],"%s.1" % params['host.gridmapfile'])

        # Run mkgridmap
        retVal = os.system("%s --safe --conf %s --output=%s" \
            % (mkgFile,confFile,params['host.gridmapfile']))

        # Check if file has been created
        if not os.path.exists(params['host.gridmapfile']):
            if os.path.exists("%s.1" % params['host.gridmapfile']):
                os.rename("%s.1" % params['host.gridmapfile'],params['host.gridmapfile'])
            glib.printWarningMessage("Failed to create/update %s" % params['host.gridmapfile'])
            glib.printWarningMessage("Any previous file has been restored")
            glib.printWarningMessage("Please run %s manually" % mkgFile)
            error_level = 2
        else:
            if retVal:
                glib.printWarningMessage("The mkgridmap script was executed with errors")
                error_level = 2                
                
        return error_level
    
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    # gLite
    if os.getenv('GLITE_LOCATION') == None:
        os.environ['GLITE_LOCATION'] = params['GLITE_LOCATION']

    if os.getenv('GLITE_LOCATION_VAR') == None:
        os.environ['GLITE_LOCATION_VAR'] = '/var/glite'
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)

    if os.getenv('GLITE_LOCATION_LOG') == None:
        os.environ['GLITE_LOCATION_LOG'] = '/var/log/glite'
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)

    if os.getenv('GLITE_LOCATION_TMP') == None:
        os.environ['GLITE_LOCATION_TMP'] = '/tmp/glite'
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    # bin and lib paths
    os.environ['PATH'] = '%s/bin:%s/externals/bin:$PATH' % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'])
    os.environ['LD_LIBRARY_PATH'] = '%s/lib:%s/externals/lib:$LD_LIBRARY_PATH' % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION'])


#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid()==0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the ConfigParams class
    params = ConfigParams()
    
    # Get an instance of the library class
    glib = gLib()
    
    # Load parameters
    if glib.loadConfiguration("%s/../glite-security-utils.cfg.xml" % glib.getScriptPath(),params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    if params.has_key('glite.installer.verbose'):
        verbose = params['glite.installer.verbose']
    else:
        verbose = 0
    glib.verbose = verbose
    
    # Set up the environment
    set_env()

    # Instantiate the service classes
    service = glite_mkgridmap()
    service.verbose = verbose
    
    # Command line opts if any
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'chv', ['checkconf', 'help', 'version', 'subservice'])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    subservice = 0

    # Check cli options
    for o, a in opts:
        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)
        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)
        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)
        if o == "--subservice":
            subservice = 1

    # Print configuration parameters
    if not verbose:
        glib.print_params(params)

    if not subservice:
        service.copyright()
        service.showVersion()
        service.banner()
        
    # Configure the service
    configuration_result = service.configure()
    if configuration_result == 0:
        print "\n%s was successfully executed\n" \
            % service.friendly_name
    elif configuration_result == 2:
        print "\nOne or more warnings were issued while executing the %s" \
            % service.friendly_name
        print "Please revise them before continuing\n"
        sys.exit(2)
    else:
        print "\nAn unrecoverable error occurred while executing the %s" \
            % service.friendly_name
        sys.exit(1)
    
